using MediatR;
using MediatR.Pipeline;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using NDB.Extensions.Swagger;
using NDB.Extensions.Swagger.Constants;
using NDB.Security.Authentication.Identity;
using NetworkResurrector.Server.Application;
using NetworkResurrector.Server.Extensions;
using Newtonsoft.Json;
using System.Reflection;

namespace NetworkResurrector.Server
{
    public class Startup
    {
        private readonly IConfiguration _configuration;

        public Startup(IConfiguration configuration)
        {
            _configuration = configuration;
        }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllers()
                .AddNewtonsoftJson(o => o.SerializerSettings.DateTimeZoneHandling = DateTimeZoneHandling.Utc);

            // Add basic authentication 
            services.AddIdentityAuthentication(_configuration.GetSection("IdentityServer")["BaseAddress"]);

            services.AddHttpContextAccessor();

            // MediatR
            services.AddMediatR(GetMediatRAssemblies());
            services.AddScoped(typeof(IPipelineBehavior<,>), typeof(RequestPreProcessorBehavior<,>));
            services.AddScoped(typeof(IPipelineBehavior<,>), typeof(RequestPostProcessorBehavior<,>));

            // AutoMapper
            services.AddAutoMapper(
                typeof(Application.Mappings.MappingProfile).Assembly);

            // Swagger
            services.AddSwagger("NetworkResurrector API", AuthorizationType.InhouseIdentity);

            // Application
            services.AddApplicationServices();

            // WakeOnLan
            services.AddWakeOnLan(_configuration);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            // global cors policy
            app.UseCors(x => x
                .AllowAnyOrigin()
                .AllowAnyMethod()
                .AllowAnyHeader());

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();
            app.UseAuthentication();
            app.UseAuthorization();
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
            app.ConfigureSwagger("NetworkResurrector API");
        }

        private Assembly[] GetMediatRAssemblies()
        {
            var assembly = typeof(Application.CommandHandlers.PingMachineHandler).Assembly;
            return new Assembly[] { assembly };
        }
    }
}
